@@ -2,11 +2,11 @@
 __all__ = [
 	'Scrambler', 'ScramblerGUI',
 	'EncryptionGUI', 'StashGUI',
-	'Instance', 'ConfigParser'
+	'Instance', 'ConfigLoader'
 ]
 
 from .scrambler import *
 from .gui.encryptiongui import *
 from .gui.stashgui import *
-from .dircrawler.instance import *
-from .utils.configparser import *
\ No newline at end of file
+from .gui.instance import *
+from .dircrawler.configloader import *
\ No newline at end of file
