@@ -1,6 +1,31 @@
-import os; import random;
+# The Scrambler
+# Copyright (c) 2022 Mystic Technology LLC
+
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+
+# The above copyright notice and this permission notice shall be included in all
+# copies or substantial portions of the Software.
+
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
 import shlex; import subprocess
+from os import remove
 from os.path import isfile, isdir, exists
+from types import NoneType
+from typing import Union
+from random import randrange
+from datetime import datetime, timedelta
 from .dircrawler.crawler import Crawler
 from .dircrawler.filemodder import FileModder
 from .utils.commoncmd import CommonCmd as cmd
@@ -8,23 +33,27 @@ from .utils.encryption import OpenSSLEncyptor as ossl
 
 class Scrambler:
 
-	def timetravel(self,path,hours_ago=None):
-		if hours_ago == None: hours_ago = random.randrange(1,121000)
+	def random_time(self) -> str:
+		end = datetime.now()
+		start = datetime.strptime('1/1/2005 12:00 AM', '%m/%d/%Y %I:%M %p')
+		delta = end - start
+		int_delta = (delta.days * 24 * 60 * 60) + delta.seconds
+		random_second = randrange(int_delta)
+		return str(start + timedelta(seconds=random_second))
 
-		pathlex = shlex.quote(path)
-		hourslex = shlex.quote(str(hours_ago))
-		command = shlex.split('touch -d "{h} hours ago" {p}'.format(h=hourslex,p=pathlex))
+	def timetravel(self, path: str) -> dict:
+		command = shlex.split('touch -d "{t}" "{p}"'.format(t=self.random_time(),p=path))
 		process = subprocess.run(command,
 					stdout=subprocess.PIPE,stderr=subprocess.PIPE)
 
 		returncode = process.returncode
 
 		if returncode == 0:
-			return {'status': 200, 'message': 'Timetravelled: ' + path}
+			return {'status': 200, 'message': 'Timetravelled: ' + str(path)}
 		else:
-			return {'status': 400, 'message': 'Error timetravelling: ' + path}
+			return {'status': 400, 'message': 'Error timetravelling: ' + str(path)}
 
-	def timetravel_files(self, dir, extension=None):
+	def timetravel_files(self, dir: str, extension: Union[str, NoneType] = None) -> dict:
 		files = Crawler.get_files(dir, extension=extension)
 
 		if len(files) <= 0: return {'status': 400, 'message': 'Error timetravelling: no files found.', 'output': []}
@@ -32,7 +61,7 @@ class Scrambler:
 		output = [self.timetravel(f)['message'] for f in files]
 		return {'status': 200, 'message': 'Timetravel files complete.', 'output': output}
 
-	def timetravel_folders(self, dir):
+	def timetravel_folders(self, dir: str) -> dict:
 		folders = Crawler.get_folders(dir)
 
 		if exists(dir) != True: return {'status': 400, 'message': 'Error timetravelling: no directory found.', 'output': []}
@@ -46,7 +75,7 @@ class Scrambler:
 
 		return {'status': 200, 'message': 'Timetravel folders complete.', 'output': output}
 
-	def invalid_stash_data(self, data):
+	def invalid_stash_data(self, data: dict) -> bool:
 		no_origin_dir = 'origin_dir' not in data
 		no_stash_dir = 'stash_dir' not in data
 		no_stash_key = 'stash_key' not in data
@@ -63,39 +92,50 @@ class Scrambler:
 		if len(keys) != len(values): return True
 		return False
 
-	def _censor_stash_errors(self, errormsg, sensitiveinfo, censored=True):
-		if censored == True:
-			return errormsg + '. Please check your config file.'
-		else:
-			return errormsg + ' ' + sensitiveinfo
-
-	def stash(self, curr_filename, curr_dir, new_filename, new_dir, overwrite=False, remove=False, censored=True):
+	def stash(self, curr_filename: str, curr_dir: str, new_filename: str, new_dir: str,
+				overwrite: bool = False, remove: bool = False, retrieve: bool = False) -> dict:
 		curr_filepath = Crawler.joinpath(curr_dir, curr_filename)
-		if exists(curr_filepath) != True: return {'status': 400,
-					'message': self._censor_stash_errors(
-						'Error: stash file was not found', curr_filepath, censored)}
+		result = {'status': None, 'message': None}
 
-		if isfile(curr_filepath) != True: return {'status': 400,
-					'message': self._censor_stash_errors(
-						'Error: invalid stash file', curr_filepath, censored)}
+		if exists(curr_filepath) != True:
+			result['status'] = 400
+			if retrieve == True:
+				result['message'] = 'Error: Retrieval failed, specified file not found in stashed directory.'
+			else:
+				result['message'] = 'Error: File not found {}'.format(curr_filepath)
+			return result
+
+		if isfile(curr_filepath) != True: 
+			result['status'] = 400
+			if retrieve == True:
+				result['message'] = 'Error: Retrieval failed, specified file in stashed directory is corrupt.'
+			else:
+				result['message'] = 'Error: File is corrupt {}'.format(curr_filepath)
+			return result
 
 		new_filepath = Crawler.joinpath(new_dir, new_filename)
-		if exists(new_filepath) == True and overwrite == False: return {'status': 400,
-					'message': self._censor_stash_errors(
-						'Error: stash file already exists', new_filepath, censored)}
+		if exists(new_filepath) == True and overwrite == False:
+			result['status'] = 400
+			if retrieve == True:
+				result['message'] = 'Error: File already exists {}'.format(new_filepath)
+			else:
+				result['message'] = 'Error: Stash failed, file already exists in stashed directory.'
+			return result
 
 		response = cmd.copyfile(curr_filepath,new_filepath)
 		if response['status'] != 200: return response
 		self.timetravel(new_filepath)
 		if remove == True: 
 			try:
-				os.remove(curr_filepath)
+				remove(curr_filepath)
 			except:
-				return {'status': 400, 'message': self._censor_stash_errors(
-						'Error: failed to remove stash file', curr_filepath, censored)}
+				if retrieve == True:
+					result['message'] = 'Error: Failed to remove file from stashed directory'
+				else:
+					result['message'] = 'Error: Failed to remove {}'.format(curr_filepath)
 		return response
 
-	def stash_all(self, data, inverse=False):
+	def stash_all(self, data: dict, retrieve: bool = False) -> dict:
 		"""
 		Data format:
 		{"origin_dir": "/home/origin_directory/",
@@ -110,10 +150,10 @@ class Scrambler:
 		stash_dir = data['stash_dir']
 
 		if isdir(origin_dir) == False: return {'status': 400,
-				'message': 'Error: could not find origin directory {}'.format(origin_dir), 'output': []}
+				'message': 'Error: Could not find origin directory {}'.format(origin_dir), 'output': []}
 
 		if isdir(stash_dir) == False: return {'status': 400,
-				'message': 'Error: could not find stash directory. Please check your config file.', 'output': []}
+				'message': 'Error: Could not find stash directory. Directory invalid or missing.', 'output': []}
 
 		stash_key = data['stash_key']
 		keys = list(stash_key.keys())
@@ -122,28 +162,30 @@ class Scrambler:
 
 		for i in range(len(keys)):
 			if keys[i] == '' or values[i] == '': return {'status': 400,
-				'message': 'Error: Invalid config file, keys and values cannot be blank. Please check your config file.',
+				'message': 'Error: Invalid config file, keys and values cannot be blank.',
 				'output': []}
 
 		for i in range(len(keys)):
-			if inverse == True:
-				response = self.stash(values[i],stash_dir,keys[i],origin_dir,overwrite=False,remove=False)
+			if retrieve == True:
+				response = self.stash(values[i],stash_dir,keys[i],origin_dir,
+										overwrite=False,remove=False,retrieve=retrieve)
 			else:
-				response = self.stash(keys[i],origin_dir,values[i],stash_dir,overwrite=True,remove=True)
+				response = self.stash(keys[i],origin_dir,values[i],stash_dir,
+										overwrite=True,remove=True,retrieve=retrieve)
 			statuscodes.append(response['status'])
 			result['output'].append(response['message'])
 
-		if inverse == True:
+		if retrieve == True:
 			keyword = 'Retrieve'
 			if len(statuscodes) > 1 and 200 not in statuscodes:
 				result['status'] = 400
-				result['message'] = 'Error: no stash files retrieved. Please check your config file.'
+				result['message'] = 'Error: No stashed files retrieved. Specified files did not exist.'
 				return result
 		else:
 			keyword = 'Stash'
 			if len(statuscodes) > 1 and 200 not in statuscodes:
 				result['status'] = 400
-				result['message'] = 'Error: no stash files found in {}'.format(origin_dir)
+				result['message'] = 'Error: No files to stash found in {}'.format(origin_dir)
 				return result
 			timetravel = self.timetravel(stash_dir)
 			if timetravel['status'] == 200:
@@ -155,24 +197,28 @@ class Scrambler:
 		result['message'] = '{} completed.'.format(keyword)
 		return result
 
-	def encrypt_msg(self, password, message, decrypt=False, pbkdf2=True):
-		data = {'medium': 'text', 'input': message, 'outpath': None}
-		result = ossl.encrypt(password, data, decrypt, pbkdf2)
+	def encrypt_msg(self, password: str, message: str, decrypt: bool = False) -> dict:
+		data = {'format': 'text', 'input': message, 'outpath': None}
+		result = ossl.encrypt(password, data, decrypt)
 		return result
 
-	def encrypt_file(self, password, filepath,
-					decrypt=False, pbkdf2=True, keep_org=False, naked=False):
+	def encrypt_file(self, password: str, filepath: str, decrypt: bool = False,
+					keep_org: bool = False, naked: bool = False) -> dict:
 		result = {'status': None, 'message': None}
-		tag_options = {'encrypt' : ['-c'],
-			'decrypt' : ['-d', '-NAKED']}
+		tag_options = {'encrypt' : ['c'],
+			'decrypt' : ['d', 'NAKED']}
 
 		if decrypt == True:
 			index = 1 if naked == True else 0
-			outpath = FileModder.add_tag(filepath,tag_options['decrypt'][index],
-								tag_options['encrypt'],tag_options['decrypt'])
+			tag = tag_options['decrypt'][index]
+			oldtags = tag_options['encrypt']
+			newtags = tag_options['decrypt']
 		else:
-			outpath = FileModder.add_tag(filepath,tag_options['encrypt'][0],
-								tag_options['decrypt'],tag_options['encrypt'])
+			tag = tag_options['encrypt'][0]
+			oldtags = tag_options['decrypt']
+			newtags = tag_options['encrypt']
+
+		outpath = FileModder.add_tag(filepath,tag,oldtags,newtags)
 
 		if exists(filepath) == False:
 			result['status'] = 400
@@ -189,11 +235,11 @@ class Scrambler:
 			result['message'] = 'Action already performed on: ' + filepath
 			return result
 
-		data = {'medium': 'file', 'input': filepath, 'outpath': outpath}
-		response = ossl.encrypt(password, data, decrypt, pbkdf2)
+		data = {'format': 'file', 'input': filepath, 'outpath': outpath}
+		response = ossl.encrypt(password, data, decrypt)
 		if response['status'] == 400:
 			try:
-				os.remove(outpath)
+				remove(outpath)
 			except:
 				pass
 			return response
@@ -206,7 +252,7 @@ class Scrambler:
 			return result
 
 		try:
-			os.remove(filepath)
+			remove(filepath)
 			result['status'] = 200
 			result['message'] = response['message'] + ' (original deleted).'
 		except:
@@ -215,12 +261,12 @@ class Scrambler:
 
 		return result
 
-	def encrypt_all_files(self, password, wd, extension=None,
-					decrypt=False, pbkdf2=True, keep_org=False, naked=False):
+	def encrypt_all_files(self, password: str, wd: str, extension: Union[str, NoneType] = None,
+					decrypt: bool = False, keep_org: bool = False, naked: bool = False) -> dict:
 		filepaths = Crawler.get_files(wd, extension=extension)
 		if len(filepaths) <= 0: return {'status': 400, 'message': 'Error: No files found.', 'output': []}
 
-		output = [self.encrypt_file(password,filepath,decrypt=decrypt,pbkdf2=pbkdf2,
+		output = [self.encrypt_file(password,filepath,decrypt=decrypt,
 					keep_org=keep_org,naked=naked)['message'] for filepath in filepaths]
 
 		timetravel = self.timetravel_folders(wd)
